import requests
import json
import io
import os
from . import __version__
from urlparse import urlunsplit, urlsplit

# Silence warnings since turning verification of against non production urls
# cause unsightly warning messages. Verification still happens against production
import requests.packages.urllib3 as urllib3
urllib3.disable_warnings()

def _url_path_join(*parts):
    """Normalize url parts and join them with a slash."""
    def first_of_each(*sequences):
        return (next((x for x in sequence if x), '') for sequence in sequences)
    schemes, netlocs, paths, queries, fragments = zip(*(urlsplit(part) for part in parts))
    scheme, netloc, query, fragment = first_of_each(schemes, netlocs, queries, fragments)
    path = '/'.join(x.strip('/') for x in paths if x)
    return urlunsplit((scheme, netloc, path, query, fragment))


class Service:
    production_base_url = "https://api.helium.com/v1"
    user_agent="Helium/"+__version__

    def __init__(self, api_key, base_url=None):
        self.api_key = api_key
        self.base_url = base_url if base_url else self.production_base_url
        self.session = requests.Session()

    def _params_from_kwargs(self, map, kwargs):
        result = {}
        for kw_key, result_key in map.iteritems():
            value = kwargs.get(kw_key, None)
            if value: result[result_key] = value
        return result

    def _mk_attributes_body(self, type, id, attributes):
        if attributes is None or not type: return None
        result = {
            "data": {
                "attributes": attributes,
                "type": type
            }
        }
        if id is not None:
            result['data']['id'] = id
        return result

    def _mk_relationships_body(self, type, ids):
        if ids is None or not type: return None
        return {
            "data": [{"id": id, "type": type} for id in ids]
        }

    def _timeseries_params_from_kwargs(self, **kwargs):
        return self._params_from_kwargs({
            "page_id": "page[id]",
            "page_size": "page[size]",
            "port": "filter[port]",
            "start": "filter[start]",
            "end": "filter[end]"
        }, kwargs)

    def _include_params_from_kwargs(self, **kwargs):
        return self._params_from_kwargs({
            "include": "include"
        }, kwargs)

    def _lua_uploads_from_files(self, files, **kwargs):
        def basename(f):
            return os.path.basename(f)
        def lua_file(name):
            return (basename(name), io.open(name, 'rb'), 'application/x-lua')

        main_file = kwargs.pop('main', None)
        # construct a dictionary of files that are not the main file
        files = { basename(name): lua_file(name) for name in files if name != main_file}
        # then add the main file if given
        if main_file:
            files["user.lua"] = lua_file(main_file)
        return files


    def _mk_url(self, path, *args):
        return _url_path_join(self.base_url, str.format(path, *args))

    def _do_url(self, method, url, params=None, json=None, files=None):
        if url is None: return None
        params = params or {}
        headers = {
            'Authorization': self.api_key,
            'User-agent': self.user_agent
        }
        req = self.session.request(method, url,
                                   params=params,
                                   json=json,
                                   files=files,
                                   headers=headers,
                                   verify=(self.base_url == self.production_base_url),
                                   allow_redirects=True)
        req.raise_for_status()
        try:
            return req.json()
        except ValueError, e:
            return req

    def _get_url(self, url,  **kwargs):
        return self._do_url('GET', url, **kwargs)

    def _post_url(self, url, **kwargs):
        return self._do_url('POST', url, **kwargs)

    def _delete_url(self, url, **kwargs):
        return self._do_url('DELETE', url, **kwargs)

    def _patch_url(self, url, **kwargs):
        return self._do_url('PATCH', url, **kwargs)

    def _get_json_path(self, json, path):
        try: # try to get the value
            return reduce(dict.__getitem__, path, json)
        except KeyError:
            return None

    def get_user(self):
        return self._get_url(self._mk_url('user'))

    def auth_user(self, user, password):
        body={
            "email":user,
            "password":password
        }
        return self._post_url(self._mk_url('user/auth'), json=body)

    def create_sensor(self, **kwargs):
        body = self._mk_attributes_body("sensor", None, kwargs)
        return self._post_url(self._mk_url('sensor'), json=body)

    def update_sensor(self, sensor_id, **kwargs):
        body=self._mk_attributes_body("sensor", sensor_id, kwargs)
        return self._patch_url(self._mk_url('sensor/{}', sensor_id), json=body)

    def delete_sensor(self, sensor_id):
        return self._delete_url(self._mk_url('sensor/{}', sensor_id))

    def get_sensors(self):
        return self._get_url(self._mk_url("sensor"))

    def get_sensor(self, sensor_id):
        return self._get_url(self._mk_url('sensor/{}', sensor_id))

    def get_sensor_timeseries(self, sensor_id, **kwargs):
        params = self._timeseries_params_from_kwargs(**kwargs)
        return self._get_url(self._mk_url('sensor/{}/timeseries', sensor_id),
                             params=params)

    def post_sensor_timeseries(self, sensor_id, data):
        return self._post_url(self._mk_url('sensor/{}/timeseries', sensor_id), json=data)

    def get_org_timeseries(self, **kwargs):
        params = self._timeseries_params_from_kwargs(**kwargs)
        return self._get_url(self._mk_url('organization/timeseries'), params=params)

    def get_prev_page(self, json):
        prev_url = self._get_json_path(json, ["links", "prev"])
        return self._get_url(prev_url)

    def get_next_page(self, json):
        next_url = self._get_json_path(json, ["links", "next"])
        return self._get_url(next_url)

    def create_label(self, name=None):
        body = self._mk_attributes_body("label", None, {
            "name": name
        }) if name else None
        return self._post_url(self._mk_url('label'), json=body)

    def delete_label(self, label_id):
        return self._delete_url(self._mk_url('label/{}', label_id))

    def get_labels(self, **kwargs):
        params = self._include_params_from_kwargs(**kwargs)
        return self._get_url(self._mk_url('label'), params=params)

    def get_label(self, label_id, **kwargs):
        params = self._include_params_from_kwargs(**kwargs)
        return self._get_url(self._mk_url('label/{}', label_id), params=params)

    def get_label_sensors(self, label_id):
        return self._get_url(self._mk_url('label/{}/relationships/sensor', label_id))

    def update_label_sensors(self, label_id, sensor_ids):
        body = self._mk_relationships_body("sensor", sensor_ids)
        return self._patch_url(self._mk_url('label/{}/relationships/sensor', label_id),
                               json=body)

    def get_elements(self):
        return self._get_url(self._mk_url('element'))

    def get_element(self, element_id):
        return self._get_url(self._mk_url('element/{}', element_id))

    def get_sensor_scripts(self):
        return self._get_url(self._mk_url('sensor-script'))

    def get_sensor_script(self, script_id):
        return self._get_url(self._mk_url('sensor-script/{}', script_id))

    def deploy_sensor_script(self, files, **kwargs):
        manifest = {
            "target": {
                "labels": kwargs.pop('labels', []),
                "sensors": kwargs.pop('sensors', [])
            }
        }
        uploads = self._lua_uploads_from_files(files, **kwargs)
        uploads['manifest'] = ('manifest.json', json.dumps(manifest), 'application/json')
        return self._post_url(self._mk_url('sensor-script'), files=uploads)


    def get_cloud_scripts(self):
        return self._get_url(self._mk_url('cloud-script'))

    def get_cloud_script(self, script_id):
        return self._get_url(self._mk_url('cloud-script/{}', script_id))

    def get_cloud_script_timeseries(self, script_id, **kwargs):
        params = self._timeseries_params_from_kwargs(**kwargs)
        return self._get_url(self._mk_url('cloud-script/{}/timeseries', script_id),
                             params=params)

    def delete_cloud_script(self, script_id):
        return self._delete_url(self._mk_url('cloud-script/{}', script_id))

    def update_cloud_script(self, script_id, name=None, start=False):
        body = self._mk_attributes_body("cloud-script", script_id, {
            "state": "running" if start else "stopped",
            "name": name
        })
        return self._patch_url(self._mk_url('cloud-script/{}', script_id),
                               json=body)

    def deploy_cloud_script(self, files, **kwargs):
        uploads = self._lua_uploads_from_files(files, **kwargs)
        return self._post_url(self._mk_url('cloud-script'), files=uploads)



import dpath.util as dpath
import urlparse
import requests
import click
import helium
import util
import timeseries as ts

pass_service=click.make_pass_decorator(helium.Service)

@click.group()
def cli():
    """Operations on cloud-scripts
    """
    pass

def _tabulate(result):
    def _map_script_filenames(json):
        files = dpath.get(json, 'meta/scripts')
        return ','.join(_extract_script_filenames(files))

    util.output(util.tabulate(result, [
        ('id', 'id'),
        ('state', 'attributes/state'),
        ('name', 'attributes/name'),
        ('files', _map_script_filenames),
        ('error', 'attributes/error/error')
    ]))


def _extract_script_filenames(files):
    return [urlparse.urlsplit(url).path.split('/')[-1] for url in files]


@cli.command()
@pass_service
def list(service):
    """List all known cloud-scripts.

    Lists all cloud-scripts for the organization.
    """
    _tabulate(service.get_cloud_scripts().get('data'))

@cli.command()
@click.argument('script')
@click.option('--name',
              help="the new name for the script")
@pass_service
def start(service, script):
    """Starts a given cloud-script.

    Starts the SCRIPT with the given id.
    """
    _tabulate([service.update_cloud_script(script, name=name, start=True).get('data')])

@cli.command()
@click.argument('script')
@click.option('--name',
              help="the new name for the script")
@pass_service
def stop(service, script, name):
    """Stop a given cloud-script.

    Stops the SCRIPT with the given id.
    """
    _tabulate([service.update_cloud_script(script, name=name, start=False).get('data')])


@cli.command()
@click.argument('script')
@pass_service
def delete(service, script):
    """Delete a given cloud-script.

    Deletes the cloud-SCRIPT with the given id.
    """
    result = service.delete_cloud_script(script)
    click.echo("Deleted" if result.status_code == 204 else result)


@cli.command()
@click.argument('script')
@ts.options()
@pass_service
def timeseries(service, script, **kwargs):
    """List readings for a cloud-script.

    Lists one page of readings for a given SCRIPT.
    Readings can be filtered by PORT and by START and END date. Dates are given
    in ISO-8601 and may be one of the following forms:

    \b
    * YYYY-MM-DD - Example: 2016-05-05
    * YYYY-MM-DDTHH:MM:SSZ - Example: 2016-04-07T19:12:06Z

    """
    data = service.get_cloud_script_timeseries(script, **kwargs).get('data')
    ts.tabulate(data)


@cli.command()
@click.argument('script')
@click.argument('file')
@pass_service
def show(service, script, file):
    """Gets a script file from a given cloud-script.

    Fetches a FILE from a given cloud-SCRIPT.
    """
    json = service.get_cloud_script(script).get('data')
    file_urls = [f.encode('utf-8') for f in dpath.get(json, 'meta/scripts')]
    names = dict(zip(_extract_script_filenames(file_urls), file_urls))
    file_url = names[opts.file]
    click.echo(requests.get(file_url).text())


@cli.command()
@click.argument('file', nargs=-1)
@click.option('--main', type=click.Path(exists=True),
              help="The main file for the script")
@pass_service
def deploy(service, file, main):
    """Deploy  a cloud-script.

    Submits a deploy request of one ore more FILEs.

    If the --main option is specified the given file is used as the `user.lua`,
    i.e. the main user script for the deployment. The file may  be part of
    the list of files given to make it easier to specify wildcards

    Note: One of the given files _must_ be called user.lua if the --main option is not given.
    This file will be considered the primary script for the deploy.
    """
    deploy=service.deploy_cloud_script(file, main=main).get('data')
    _tabulate([deploy])


@cli.command()
@click.argument('script')
@pass_service
def status(service, script):
    """Displays current status information for a script.

    Display status information a given SCRIPT. If the script is in an error
    condition the error details are displayed.
    """
    data = service.get_cloud_script(script).get('data')
    click.echo('Status: ' + dpath.get(data, "attributes/state"))
    error = dpath.get(data, "attributes/error")
    if error:
        click.echo('Error: ' + error.get('details'))



import commands
import helium
import click

@click.command(cls=commands.Loader)
@click.option('--api-key',
              envvar='HELIUM_API_KEY',
              help='your Helium API key. Can also be specified using the HELIUM_API_KEY environment variable')
@click.option('--host',
              envvar='HELIUM_API_URL',
              default=None,
              help= 'The Helium base API URL. Can also be specified using the HELIUM_API_URL environment variable.' )
@click.pass_context
def cli(ctx, api_key, host):
    ctx.obj = helium.Service(api_key, host)


def main():
    import sys
    args = sys.argv[1:]
    cli.main(args=args, prog_name=None)

if __name__ == '__main__':
    main()



import sensor as _sensor
import helium
import util
import click
import dpath.util as dpath
import timeseries as ts

pass_service=click.make_pass_decorator(helium.Service)

@click.group()
def cli():
    """Operations on labels of sensors.
    """
    pass

def _tabulate(result):
    def _map_sensor_count(json):
        return len(dpath.get(json, 'relationships/sensor/data'))
    util.output(util.tabulate(result, [
        ('id', 'id'),
        ('name', 'attributes/name'),
        ('sensors', _map_sensor_count)
    ]))


@cli.command()
@click.argument('label',required=False)
@click.pass_context
def list(ctx, label):
    """Lists information on labels.

    Lists information on a given label or all labels in the organization
    """
    if label:
        ctx.invoke(_sensor.list, label=label)
    else:
        service = ctx.find_object(helium.Service)
        _tabulate(service.get_labels(include='sensor').get('data'))


@cli.command()
@click.argument('name', nargs=1)
@click.argument('sensor', nargs=-1)
@click.pass_context
def create(ctx, name, sensor):
    """Create a label.

    Creates a label with a given NAME and an (optional) list of SENSORs
    associated with that label.
    """
    service = ctx.find_object(helium.Service)
    label = service.create_label(name).get('data')
    if sensor:
        service.update_label_sensors(label['id'], sensor)
    ctx.invoke(_sensor.list, label=label['id'])


@cli.command()
@click.argument('label')
@pass_service
def delete(service, label):
    """Delete a label.

    Deletes the LABEL with the given id
    """
    result = service.delete_label(label)
    click.echo("Deleted" if result.status_code == 204 else result)


@cli.command()
@click.argument('label',nargs=1)
@click.argument('sensor',nargs=-1)
@click.pass_context
def add(ctx, label, sensor):
    """Add sensors to a label.

    Adds a given list of SENSORs to the LABEL with the given id.
    """
    service = ctx.find_object(helium.Service)
    sensors = service.get_label_sensors(label).get('data')
    sensor_ids = dpath.values(sensors, "*/id")
    sensor_ids = set(sensor_ids).union(set(sensor))
    service.update_label_sensors(label, sensor_ids)
    ctx.invoke(_sensor.list, label=label)


@cli.command()
@click.argument('label',nargs=1)
@click.argument('sensor',nargs=-1)
@click.pass_context
def remove(ctx, label, sensor):
    """Remove sensors from a label.

    Removes a given list of SENSORs from the LABEL with the given id.
    """
    service = ctx.find_object(helium.Service)
    sensors = service.get_label_sensors(label).get('data')
    sensor_ids = dpath.values(sensors, "*/id")
    sensor_ids = set(sensor_ids).difference(set(sensor))
    if sensor_ids is None: sensor_ids = []
    service.update_label_sensors(label, sensor_ids)
    ctx.invoke(_sensor.list, label=label)


@cli.command()
@click.argument('label')
@ts.format_option()
@ts.options()
@pass_service
def dump(service, label, format, **kwargs):
    """Dumps timeseries data to files.

    Dumps the timeseries data for all sensors in a given LABEL.

    One file is generated for each sensor with the sensor id as filename and the
    file extension based on the requested dump format

    This command takes the same arguments as the `timeseries` command, including
    the ability to filter by PORT, START and END date
    """
    sensors = dpath.values(service.get_sensors(), '/data/*/id')
    ts.dump(service, sensors, format, **kwargs)
